import {Component, Input, ViewChild,} from '@angular/core';
import {MatDialog} from "@angular/material/dialog";
import {BeforeLeaveGuard} from "../../../window/sidenav/navigation-on-leave-guard";
import {MatPaginator, PageEvent} from "@angular/material/paginator";
import {GroupRo} from "../../../common/model/group-ro.model";
import {ResourceRo} from "../../../common/model/resource-ro.model";
import {AlertMessageService} from "../../../common/alert-message/alert-message.service";
import {EditGroupService} from "../edit-group.service";
import {finalize} from "rxjs/operators";
import {TableResult} from "../../../common/model/table-result.model";
import {ConfirmationDialogComponent} from "../../../common/dialogs/confirmation-dialog/confirmation-dialog.component";
import {ResourceDialogComponent} from "./resource-dialog/resource-dialog.component";
import {DomainRo} from "../../../common/model/domain-ro.model";
import {ResourceDefinitionRo} from "../../../system-settings/admin-extension/resource-definition-ro.model";
import {VisibilityEnum} from "../../../common/enums/visibility.enum";
import {
  ManageMembersDialogComponent
} from "../../../common/dialogs/manage-members-dialog/manage-members-dialog.component";
import {MemberTypeEnum} from "../../../common/enums/member-type.enum";
import {TranslateService} from "@ngx-translate/core";
import {lastValueFrom} from "rxjs";


@Component({
  selector: 'group-resource-panel',
  templateUrl: './group-resource-panel.component.html',
  styleUrls: ['./group-resource-panel.component.scss']
})
export class GroupResourcePanelComponent implements BeforeLeaveGuard {

  title: string = "";
  private _group: GroupRo;
  @Input() domain: DomainRo;
  @Input() domainResourceDefs: ResourceDefinitionRo[];
  displayedColumns: string[] = ['identifierValue', 'identifierScheme'];
  data: ResourceRo[] = [];
  selected: ResourceRo;
  filter: any = {};
  resultsLength = 0;
  isLoadingResults = false;
  @ViewChild(MatPaginator) paginator: MatPaginator;

  constructor(private editGroupService: EditGroupService,
              private alertService: AlertMessageService,
              private dialog: MatDialog,
              private translateService: TranslateService) {
  }


  @Input()
  set group(value: GroupRo) {
    (async () => {
      if (this._group == value) {
        return;
      }
      this._group = value;
      this.title = await lastValueFrom(this.translateService.get("group.resource.panel.title", {groupName: (!!this._group ? ": [" + this._group.groupName + "]" : "")}));
      if (!!this._group) {
        this.loadGroupResources();
      } else {
        this.isLoadingResults = false;
      }
    })();
  }

  get group(){
    return this._group;
  }

  onPageChanged(page: PageEvent) {
    this.loadGroupResources();
  }

  loadGroupResources() {

    this.onResourceSelected(null);
    if (!this._group) {
      return;
    }

    this.isLoadingResults = true;
    this.editGroupService.getGroupResourcesForGroupAdminObservable(this._group,this.domain, this.filter, this.paginator.pageIndex, this.paginator.pageSize)
      .pipe(
        finalize(() => {
          this.isLoadingResults = false;
        }))
      .subscribe((result: TableResult<ResourceRo>) => {
          this.data = [...result.serviceEntities];
          this.resultsLength = result.count;
          this.isLoadingResults = false;
        }
      );
  }


  applyResourceFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.filter["filter"] = filterValue.trim().toLowerCase();
    this.refresh();
  }

  get createResourceDisabled(): boolean {
    return !this._group;
  }

  public onCreateResourceButtonClicked() {
    this.showResourceEditDialog(this.createResource());
  }

  createResource ():ResourceRo {
    return {
      resourceTypeIdentifier: !!this.domainResourceDefs && this.domainResourceDefs.length>0 ? this.domainResourceDefs[0].identifier:"",
      identifierValue: "",
      smlRegistered: false,
      visibility: VisibilityEnum.Public
    }
  }

  public refresh() {
    if (this.paginator) {
      this.paginator.firstPage();
    }
    this.loadGroupResources();
  }

  async onEditSelectedGroupMembersButtonClicked() {
    this.dialog.open(ManageMembersDialogComponent, {
      data: {
        membershipType: MemberTypeEnum.RESOURCE,
        domain: this.domain,
        group: this._group,
        resource: this.selected,
        formTitle: await lastValueFrom(this.translateService.get("group.resource.panel.manage.members.dialog.title"))
      }
    }).afterClosed().subscribe(value => {
      this.refresh();
    });
  };
  public onEditSelectedButtonClicked() {
    this.showResourceEditDialog(this.selected)
  }
  public async showResourceEditDialog(resource: ResourceRo) {
    this.dialog.open(ResourceDialogComponent, {
      data: {
        resource: resource,
        group: this._group,
        domain: this.domain,
        domainResourceDefs: this.domainResourceDefs,
        formTitle: await lastValueFrom(this.translateService.get("group.resource.panel.resource.details.dialog.title"))
      }
    }).afterClosed().subscribe(value => {
      this.refresh();
    });
  }

  public async onDeleteSelectedButtonClicked() {
    if (!this._group || !this._group.groupId) {
      this.alertService.error(await lastValueFrom(this.translateService.get("group.resource.panel.error.delete.group")));
      return;
    }

    if (!this.selected || !this.selected.resourceId) {
      this.alertService.error(await lastValueFrom(this.translateService.get("group.resource.panel.error.delete.resource")));
      return;
    }

    this.dialog.open(ConfirmationDialogComponent, {
      data: {
        title: await lastValueFrom(this.translateService.get("group.resource.panel.delete.confirmation.dialog.title")),
        description: await lastValueFrom(this.translateService.get("group.resource.panel.delete.confirmation.dialog.description", {
          identifierScheme: this.selected.identifierScheme,
          identifierValue: this.selected.identifierValue
        }))
      }
    }).afterClosed().subscribe(result => {
      if (result) {
        this.deleteResource(this.group, this.selected);
      }
    });
  }

  deleteResource(group: GroupRo, resource: ResourceRo) {
    this.isLoadingResults = true;
    this.editGroupService.deleteResourceFromGroup(resource, this._group, this.domain)
      .pipe(
        finalize(() => {
          this.refresh();
          this.isLoadingResults = false;
        }))
      .subscribe(async (result: ResourceRo) => {
          if (result) {
            this.alertService.success(await lastValueFrom(this.translateService.get("group.resource.panel.success.delete", {
              identifierScheme: this.selected.identifierScheme,
              identifierValue: this.selected.identifierValue
            })));
          }
        }, (error)=> {
          this.alertService.error(error.error?.errorDescription);
        }
      );
  }


  public onResourceSelected(resource: ResourceRo) {
    this.selected = resource;
  }

  get disabledForm(): boolean {
    return !this._group;
  }

  isDirty(): boolean {
    return false
  }
}






